/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.sys.startstopp.prozessvew;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.startstopp.skriptvew.GlobaleDaten;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVerwaltung;
import de.bsvrz.sys.startstopp.skriptvew.StartStoppApp;
import de.bsvrz.sys.startstopp.skriptvew.StartStoppVerwaltung;
import de.bsvrz.sys.startstopp.skriptvew.XMLToolsTest;

/**
 * Klasse zum Testen des Empfangs und des Auswertens der Attributgruppe 'atg.startStoppAnfrage'
 * in den unterschiedlichen Varianten
 * @author Dambach Werke GmbH
 */
public class StartStoppAnfrageTest
{
  /**
   * Name einer temp. Datei 
   */
  private static String m_datei        = "prozessverwaltungTest.xml";
  
  /**
   * Datenverteilerverbindung
   */
  private static ClientDavInterface m_connection = null;

  /**
   * Prozessverwaltung
   */
  private static ProzessVerwaltung m_pv;
  
  /**
   * Instanz der Klasse DaVKommunikation
   */
  private DaVKommunikation m_dav = DaVKommunikation.getInstanz();
  
  /**
   * Merker, dass zu prfender Datensatz empfangen wurde
   */
  private boolean m_empfangen = false;

  /**
   * ID des gestarteten Prozesses (Variante 4)
   */
  private String m_prozessId;
  
  /**
   * Hilfmerker zum Prfen der Variante 4 
   */
  private int m_warteArt;
  
  /**
   * StartStopp Antwort empfangen
   */
  private boolean m_ok;
  
  private long m_counter;

  /**
   * Liste mit den Namen der erwarteten Inkarnationen
   */
  private List<String> m_erwarteteNamen = Arrays.asList ( "Betriebsmeldungsverwaltung",
                                                          "UeberpruefungSystem",
                                                          "Applikation1.1",
                                                          "Applikation1.2",
                                                          "Parametrierung",
                                                          "Applikation1.4",
                                                          "Applikation1.3",
                                                          "Konfiguration",
                                                          "Datenverteiler"
                                                        );

  /** 
   * Wartezeit die aktiviert werden kann, um die Prozessinformationen zustzlich am GTM
   * auszuwerten
   */
  private long m_warteZeit = 0;
  
  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    // Zugangsdaten DaV
    
    String rechner  = "localhost";
    int port        = 8083;
    String benutzer = "Tester";
    String password = "geheim";
    
    // Globale Einstellungen die fr die Klasse DaVKommunikation bentigt werden
    
    GlobaleDaten glob = GlobaleDaten.getInstanz();
    
    glob.setDavAdresse( rechner );
    glob.setDavPort( port );
    glob.setDavBenutzer( benutzer );
    glob.setDavPasswort( password );

    // Notwendige Initialisierungen
    
    String[] args = new String [2];
    
    args [0] = "-startStoppKonfiguration=C:/TestStartStopp"; 
    args [1] = "-_versionieren=C:/TestStartStopp/StartStopp.xml";
    
    StartStoppApp._applicationLabel = StartStoppApp.createApplicationLabel( args );
    ProzessVerwaltung.getInstanz().setKernSystemGestartet( true );
    
    // Datenverteilerverbindung aufbauen zum Prfen der gesendeten Datenstze
    
    ClientDavParameters parameters = new ClientDavParameters();
    
    parameters.setDavCommunicationAddress( rechner );
    parameters.setDavCommunicationSubAddress( port );
    parameters.setUserName( benutzer );
    parameters.setUserPassword( password );

    m_connection = new ClientDavConnection(parameters);
    m_connection.connect();
    m_connection.login();
    
    // Testumgebung initialisieren
    
    StartStoppApp a = new StartStoppApp ();
    
    a.setStartStoppKonfiguration( "C:\\JUnit");
    a.setStartStoppInkarnationsName( "StartStopp" );
    a.bestimmeRechnerAdresse();
    
    // Prozessverwaltung
    
    m_pv = ProzessVerwaltung.getInstanz();

  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
    m_connection.disconnect( false, "" );
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Methode prft den korrekten Empfang einer StartStoppAnfrage Variante 1
   * Die Prfung erfolgt in dem die Daten die in der Resource ../xml/startStopp_2.xml
   * gespeichert sind ber die Attributgruppe startStoppAnfrage Variante 1 bertragen werden 
   * und anschliessend geprft wird ob alle Inkarnationen in der StartStopp Applikation
   * eingetragen sind. 
   */
  @Test
  public void testSendeStartStoppAnfrageV1()
  {
    // Initialisierung
    
    StartStoppApp.setTestBetrieb( true );  // Damit Applikationen nicht gestartet werden
    
    m_pv.reset();
    StartStoppVerwaltung.getInstanz().reset();
    GlobaleDaten.getInstanz().removeAllListener();

    // Verbindung zum DaV aufbauen
    
    if (!m_dav.baueVerbindungDaVAuf())
      fail( "Verbindungsaufbau nicht mglich" );

    m_dav.anmeldeDaV();

    // XML-String aus Resource einlesen
    
    XMLToolsTest test = new XMLToolsTest ();
    String xml = test.einlesenAusResource( "../xml/startStopp_2.xml" );
    test.schreibeInDatei( m_datei, xml );
    
    // Warten auf StartStoppAntwort
    
    // Von JUNIT aus am DaV anmelden und die empfangene Prozessinformationen auswerten
    
    final Object sync = new Object();

    m_empfangen = false;
    
    final String atg1 = "atg.startStoppAntwort";
    m_ok = false;
    m_counter = 0;
    
    // Neuen Empfnger anlegen der auf die Attributgruppe wartet
    
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update( ResultData[] results )
      {
        for (int i = 0; i < results.length; i++) 
        {
          ResultData dat = results [i];

          // Auslesen der Attributgruppe

          if (dat.getDataDescription().getAttributeGroup().getPid().equals( atg1 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              String fehler = dat.getData().getTextValue( "fehler" ).getText();
              if (fehler.equals( "kein Fehler" ))
              {
                m_counter++;
                if (m_counter == m_erwarteteNamen.size())
                  m_ok = true;
              }
            }
          }
        }

        if (m_ok)
        {
          m_empfangen = true;
          
          synchronized (sync)
          {
            sync.notify();
          }
        }
      }
    };

    // Anmelden zum Empfangen der zu prfenden Attributgruppe
    
    AttributeGroup atg  = m_connection.getDataModel().getAttributeGroup( atg1 );
    Aspect asp          = m_connection.getDataModel().getAspect("asp.antwort");

    long id = m_connection.getLocalApplicationObject().getId();
    SystemObject sysObj = m_connection.getDataModel().getObject(id);

    DataDescription dd = new DataDescription(atg, asp);
  
    m_connection.subscribeReceiver(empf, sysObj, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

    // StartStoppAnfrage senden
    
    sendeStartStoppAnfrageV1( "starten", "", "", "0" );
    
    // Warten auf Antwort des ClientReceiverInterface
    // hier muss nicht gewartet werden, da die Antworten schon da sein mssen !
    
    // Sleep wird eingebaut damit man die Prozessinformationen mit dem GTM sehen kann
    
    mySleep (m_warteZeit );

    // Verbindung zum DaV wieder trennen
    
    m_dav.allesAbmelden();
    m_dav.trenneDaV();
    
    m_connection.unsubscribeReceiver( empf, sysObj, dd);

    if (!m_empfangen)
      fail ("StartStoppAntwort nicht korrekt empfangen " + m_counter);
    
    pruefeInkarnationsNamen();
    
    StartStoppApp.setTestBetrieb( false );  // Wieder zurckschalten
  }

  /**
   * Methode prft den korrekten Empfang einer StartStoppAnfrage Variante 2
   * Die Prfung erfolgt in dem die Daten die in der Resource ../xml/startStopp_2.xml
   * gespeichert sind ber die Attributgruppe startStoppAnfrage Variante 2 als XML-Struktur 
   * bertragen werden und anschliessend geprft wird ob alle Inkarnationen in der StartStopp 
   * Applikation eingetragen sind. 
   */
  @Test
  public void testSendeStartStoppAnfrageV2()
  {
    // Initialisierung
    
    m_pv.reset();
    StartStoppVerwaltung.getInstanz().reset();
    GlobaleDaten.getInstanz().removeAllListener();

    // Verbindung zum DaV aufbauen
    
    if (!m_dav.baueVerbindungDaVAuf())
      fail( "Verbindungsaufbau nicht mglich" );

    m_dav.anmeldeDaV();

    // XML-String aus Resource einlesen
    
    XMLToolsTest test = new XMLToolsTest ();
    String xml = test.einlesenAusResource( "../xml/startStopp_2.xml" );
    
    // StartStoppAnfrage senden
    
    sendeStartStoppAnfrageV2( "starten", "", xml, "0" );
    
    // Warten auf StartStoppAntwort
    
    // Von JUNIT aus am DaV anmelden und die empfangene Prozessinformationen auswerten
    
    final Object sync = new Object();

    m_empfangen = false;
    
    final String atg1 = "atg.startStoppAntwort";
    m_ok = false;
    
    // Neuen Empfnger anlegen der auf die Attributgruppe wartet
    
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update( ResultData[] results )
      {
        for (int i = 0; i < results.length; i++) 
        {
          ResultData dat = results [i];

          // Auslesen der Attributgruppe

          if (dat.getDataDescription().getAttributeGroup().getPid().equals( atg1 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              String fehler = dat.getData().getTextValue( "fehler" ).getText();
              if (fehler.equals( "kein Fehler" ))
                m_ok = true;
            }
          }
        }

        if (m_ok)
        {
          m_empfangen = true;
          
          synchronized (sync)
          {
            sync.notify();
          }
        }
      }
    };

    // Anmelden zum Empfangen der zu prfenden Attributgruppe
    
    AttributeGroup atg  = m_connection.getDataModel().getAttributeGroup( atg1 );
    Aspect asp          = m_connection.getDataModel().getAspect("asp.antwort");

    long id = m_connection.getLocalApplicationObject().getId();
    SystemObject sysObj = m_connection.getDataModel().getObject(id);

    DataDescription dd = new DataDescription(atg, asp);
  
    m_connection.subscribeReceiver(empf, sysObj, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

    // Warten auf Antwort des ClientReceiverInterface
    
    synchronized (sync)
    {
      try
      {
        sync.wait(30000);
      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }
    }

    // Sleep wird eingebaut damit man die Prozessinformationen mit dem GTM sehen kann
    
    mySleep (m_warteZeit );

    // Verbindung zum DaV wieder trennen
    
    m_dav.allesAbmelden();
    m_dav.trenneDaV();
    
    m_connection.unsubscribeReceiver( empf, sysObj, dd);

    if (!m_empfangen)
      fail ("StartStoppAntwort nicht korrekt empfangen");
    
    pruefeInkarnationsNamen(); 
  }

  /**
   * Methode prft den korrekten Empfang einer StartStoppAnfrage Variante 3
   * Die Prfung erfolgt in dem die Daten die in der Resource ../xml/startStopp_2.xml
   * gespeichert sind ber die Attributgruppe startStoppAnfrage Variante 3 als Datei 
   * bertragen werden und anschliessend geprft wird ob alle Inkarnationen in der StartStopp 
   * Applikation eingetragen sind. 
   */
  @Test
  public void testSendeStartStoppAnfrageV3()
  {
    // Initialisierung
    
    m_pv.reset();
    StartStoppVerwaltung.getInstanz().reset();
    GlobaleDaten.getInstanz().removeAllListener();

    // Verbindung zum DaV aufbauen
    
    if (!m_dav.baueVerbindungDaVAuf())
      fail( "Verbindungsaufbau nicht mglich" );

    m_dav.anmeldeDaV();

    // XML-String aus Resource einlesen
    
    XMLToolsTest test = new XMLToolsTest ();
    String xml = test.einlesenAusResource( "../xml/startStopp_2.xml" );
    test.schreibeInDatei( m_datei, xml );
    
    // StartStoppAnfrage senden
    
    sendeStartStoppAnfrageV3( "starten", "", m_datei, "0" );
    
    // Warten auf StartStoppAntwort
    
    // Von JUNIT aus am DaV anmelden und die empfangene Prozessinformationen auswerten
    
    final Object sync = new Object();

    m_empfangen = false;
    
    final String atg1 = "atg.startStoppAntwort";
    m_ok = false;
    
    // Neuen Empfnger anlegen der auf die Attributgruppe wartet
    
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update( ResultData[] results )
      {
        for (int i = 0; i < results.length; i++) 
        {
          ResultData dat = results [i];

          // Auslesen der Attributgruppe

          if (dat.getDataDescription().getAttributeGroup().getPid().equals( atg1 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              String fehler = dat.getData().getTextValue( "fehler" ).getText();
              if (fehler.equals( "kein Fehler" ))
                m_ok = true;
            }
          }
        }

        if (m_ok)
        {
          m_empfangen = true;
          
          synchronized (sync)
          {
            sync.notify();
          }
        }
      }
    };

    // Anmelden zum Empfangen der zu prfenden Attributgruppe
    
    AttributeGroup atg  = m_connection.getDataModel().getAttributeGroup( atg1 );
    Aspect asp          = m_connection.getDataModel().getAspect("asp.antwort");

    long id = m_connection.getLocalApplicationObject().getId();
    SystemObject sysObj = m_connection.getDataModel().getObject(id);

    DataDescription dd = new DataDescription(atg, asp);
  
    m_connection.subscribeReceiver(empf, sysObj, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

    // Warten auf Antwort des ClientReceiverInterface
    
    synchronized (sync)
    {
      try
      {
        sync.wait(30000);
      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }
    }

    // Sleep wird eingebaut damit man die Prozessinformationen mit dem GTM sehen kann
    
    mySleep (m_warteZeit );

    // Verbindung zum DaV wieder trennen
    
    m_dav.allesAbmelden();
    m_dav.trenneDaV();
    
    m_connection.unsubscribeReceiver( empf, sysObj, dd);

    if (!m_empfangen)
      fail ("StartStoppAntwort nicht korrekt empfangen");
    
    pruefeInkarnationsNamen();
  }

  /**
   * Methode prft den korrekten Empfang einer StartStoppAnfrage Variante 4
   * Die Prfung erfolgt in dem in die Skriptverwaltung eine StartStopp Datei mit einer Applikation
   * 'Taschenrechner' geladen wird die manuell gestartet wird. Diese Applikation wird dann ber
   * ein Telegramme Variante 4 gestartet. Nach Empfang der StartStopp Antwort wird die Applikation
   * ber ein erneutes Telegramm Variante 4 wieder beendet.
   */
  @Test
  public void testSendeStartStoppAnfrageV4()
  {
    m_pv.reset();
    StartStoppVerwaltung.getInstanz().reset();
    GlobaleDaten.getInstanz().removeAllListener();

    // Verbindung zum DaV aufbauen
    
    if (!m_dav.baueVerbindungDaVAuf())
      fail( "Verbindungsaufbau nicht mglich" );

    m_dav.anmeldeDaV();
    
    // Skriptverwaltung anlegen

    XMLToolsTest test = new XMLToolsTest ();
    String xmlAlsString = test.einlesenAusResource( "../xml/startStoppCalc.xml" );
    test.schreibeInDatei( m_datei, xmlAlsString );
    
    SkriptVerwaltung sv = new SkriptVerwaltung (m_datei, 1, 0);
    
    if (sv.isFehlerStartStopp())
      fail ( sv.getFehlerText() );

    List<ProzessDaten> pd = m_pv.getAlleProzessDaten();
    
    Iterator<ProzessDaten> it = pd.iterator();
    while (it.hasNext())
    {
      ProzessDaten d = it.next();
      System.out.println(d.getName());
    }

    // Warten auf StartStoppAntwort
    
    // Von JUNIT aus am DaV anmelden und die empfangene Prozessinformationen auswerten
    
    final Object sync = new Object();

    m_empfangen = false;
    
    final String atg1 = "atg.startStoppAntwort";
    m_ok = false;
    
    // Neuen Empfnger anlegen der auf die Attributgruppe wartet
    
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update( ResultData[] results )
      {
        for (int i = 0; i < results.length; i++) 
        {
          ResultData dat = results [i];

          // Auslesen der Attributgruppe

          if (dat.getDataDescription().getAttributeGroup().getPid().equals( atg1 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              String zustand = dat.getData().getTextValue( "zustand" ).getText();
              String fehler  = dat.getData().getTextValue( "fehler" ).getText();
              m_prozessId    = dat.getData().getTextValue( "id" ).getText();
              
//              System.out.println("ID " + m_prozessId + " Zustand " + zustand + " Fehler " + fehler);
              
              // Warten auf Starten der Applikation
              
              if (m_warteArt == 0)
              {
                if (fehler.equals( "kein Fehler" ) &&
                    zustand.equals( "gestartet"  ))
                  m_ok = true;
              }
              
              if (m_warteArt == 1)
              {
                if (fehler.equals( "kein Fehler" ) &&
                    zustand.equals( "gestoppt"  ))
                  m_ok = true;
              }
            }
          }
        }

        if (m_ok)
        {
          m_empfangen = true;
          
          synchronized (sync)
          {
            sync.notify();
          }
        }
      }
    };

    // Anmelden zum Empfangen der zu prfenden Attributgruppe
    
    AttributeGroup atg  = m_connection.getDataModel().getAttributeGroup( atg1 );
    Aspect asp          = m_connection.getDataModel().getAspect("asp.antwort");

    long id = m_connection.getLocalApplicationObject().getId();
    SystemObject sysObj = m_connection.getDataModel().getObject(id);

    DataDescription dd = new DataDescription(atg, asp);
  
    m_connection.subscribeReceiver(empf, sysObj, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

    // StartStoppAnfrage senden 
    // --> Starten der Applikation mit Name Taschenrechner
    
    m_warteArt = 0;
    m_prozessId = "";
    m_empfangen = false;
    
    sendeStartStoppAnfrageV4( "starten", "", "Taschenrechner", "0" );
    
    // Warten auf Antwort des ClientReceiverInterface
    
    synchronized (sync)
    {
      try
      {
        sync.wait(30000);
      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }
    }

    // Sleep wird eingebaut damit der Prozess auch tatschlich gestartet wird (sonst wird Testprogramm
    // zu frh beendet)
    
    mySleep (5000 );

    // StartStoppAnfrage senden
    // --> stoppen der Applikation
    
    if (m_empfangen)
    {
      m_warteArt = 1;
      m_empfangen = false;

      sendeStartStoppAnfrageV4( "stoppen", m_prozessId, "", "0" );
    
      // Warten auf Antwort des ClientReceiverInterface
      
      synchronized (sync)
      {
        try
        {
          sync.wait(30000);
        }
        catch (InterruptedException e)
        {
          e.printStackTrace();
        }
      }
  
      // Sleep wird eingebaut damit der Prozess auch tatschlich beendet wird (sonst wird Testprogramm
      // zu frh beendet)
  
      mySleep (5000 );
    }

    // Verbindung zum DaV wieder trennen
    
    m_dav.allesAbmelden();
    m_dav.trenneDaV();

    m_connection.unsubscribeReceiver( empf, sysObj, dd);

    if (!m_empfangen)
      fail ("StartStoppAntwort nicht korrekt empfangen");
  }

  /**
   * Methode zum Senden einer StartStoppAnfrage Variante 1
   * @param aktion Aktion
   * @param prozessID ProzessID
   * @param xmlDatei Name der XML-Datei
   * @param simulationsVariante Simulationsvariante
   */
  private void sendeStartStoppAnfrageV1( String aktion, String prozessID, String xmlDatei, String simulationsVariante )
  {
    m_pv.reset();
    
    // Skriptverwaltung anlegen, aus den Inkarnationen dieser Skripverwaltung wird die
    // StartStoppAnfrage Variante 1 gebildet

    XMLToolsTest test = new XMLToolsTest ();
    String xml = test.einlesenAusResource( "../xml/startStopp_2.xml" );
    test.schreibeInDatei( m_datei, xml );
    
    SkriptVerwaltung sv = new SkriptVerwaltung (m_datei, 1234, 0);
    List<ProzessDaten> testDaten = m_pv.getAlleProzessDaten();
    m_pv.reset();

    //  zu versendende Data Informationen
    
    AttributeGroup atg = (AttributeGroup) m_connection.getDataModel().getAttributeGroup("atg.startStoppAnfrage");   

    for (int a = 0; a<testDaten.size(); ++a)
    {
      Data data = m_connection.createData(atg );
      
      if (data != null)
      {
        data.getReferenceValue( "absenderId" ).setText( new String("" + m_connection.getLocalApplicationObject().getId()) );
        data.getTextValue( "absenderZeichen" ).setText( new String("Absender Zeichen XYZ") );
        
        data.getArray( "V3" ).setLength(0);
        data.getArray( "V2" ).setLength(0);
        data.getArray( "V4" ).setLength(0);
        data.getArray( "V1" ).setLength(1);
  
        for (int i=0; i<1; ++i)
        {
          ProzessDaten aktuell = testDaten.get( a );        
        
          Data d = data.getArray( "V1" ).getItem( i );
          d.getScaledValue("Aktion").setText( "starten" );
        
          d.getTextValue( "ProzessID").setText("" );
          d.getTextValue( "Name" ).setText( aktuell.getName()  );
          d.getTextValue( "AusfuehrbareDatei" ).setText( aktuell.getAusfuehrbareDatei() );
      
          // Aufrufparameter
          
          List<String> ap = aktuell.getAufrufParameter();
          d.getArray( "AufrufParameter").setLength(ap.size());
          for (int j=0; j<ap.size(); ++j)
            d.getArray("AufrufParameter").getItem(j).getTextValue( "AufrufparameterWert").setText( ap.get( j ) );
        
          // StartArt
    
          d.getItem( "StartArt" ).getScaledValue( "OptionStart" ).setText( aktuell.getStartArt().getOption().getText());
          d.getItem( "StartArt" ).getScaledValue( "NeuStart" ).setText( aktuell.getStartArt().getNeuStart().getText());
          d.getItem( "StartArt" ).getTextValue  ( "Intervall" ).setText( aktuell.getStartArt().getIntervallZeit());
        
          // StartBedingung
          
          List<StartBedingung> stab = aktuell.getStartBedingung();
          
          // StandardAusgabe
  
          d.getItem( "StandardAusgabe").getScaledValue( "OptionStandardAusgabe" ).setText( aktuell.getStandardAusgabe().getOption().getText() );
          d.getItem( "StandardAusgabe").getTextValue( "Datei").setText( aktuell.getStandardAusgabe().getDateiAlias() );
          
          // FehlerAusgabe
  
          d.getItem( "FehlerAusgabe").getScaledValue( "OptionFehlerAusgabe").setText( aktuell.getStandardFehlerAusgabe().getOption().getText());
          d.getItem( "FehlerAusgabe").getTextValue( "Datei" ).setText (aktuell.getStandardFehlerAusgabe().getDateiAlias());
          
          // StartVerhalten Fehler
          
          d.getItem( "StartVerhaltenFehler").getScaledValue( "StartVerhaltenFehlerOption").setText( "beenden" );
          d.getItem( "StartVerhaltenFehler").getScaledValue( "Wiederholrate").set( 0);
          
          // StoppVerhaltenFehler
  
          d.getItem( "StoppVerhaltenFehler").getScaledValue( "StoppVerhaltenFehlerOption").setText( "Abbruch" );
          d.getItem( "StoppVerhaltenFehler").getScaledValue( "Wiederholrate").set(  0 );
          
          sendeDaten( data );
          mySleep( 1000 );
        }
      }
    }
  }

  /**
   * Methode zum Senden einer StartStoppAnfrage Variante 2
   * @param aktion Aktion
   * @param prozessID ProzessID
   * @param xml XML String
   * @param simulationsVariante Simulationsvariante
   */
  private void sendeStartStoppAnfrageV2( String aktion, String prozessID, String xml, String simulationsVariante )
  {
    AttributeGroup atg = (AttributeGroup) m_connection.getDataModel().getAttributeGroup("atg.startStoppAnfrage");   
    
    Data data = m_connection.createData(atg );
    
    if (data != null)
    {
      data.getReferenceValue( "absenderId" ).setText( new String("" + m_connection.getLocalApplicationObject().getId()) );
      data.getTextValue( "absenderZeichen" ).setText( new String("Absender Zeichen XYZ") );
  
      data.getArray( "V1" ).setLength(0);
      data.getArray( "V3" ).setLength(0);
      data.getArray( "V4" ).setLength(0);
      data.getArray( "V2" ).setLength(1);
      data.getArray( "V2" ).getItem( 0 ).getScaledValue("Aktion").setText( aktion ); 
      data.getArray( "V2" ).getItem( 0 ).getTextValue( "StartStoppID").setText(prozessID );
      data.getArray( "V2" ).getItem( 0 ).getScaledValue( "SimulationsVariante" ).set( Long.parseLong(simulationsVariante)  );
      data.getArray( "V2" ).getItem( 0 ).getTextValue( "xml" ).setText( xml);
      
      sendeDaten( data );
    }
  }

  /**
   * Methode zum Senden einer StartStoppAnfrage Variante 3
   * @param aktion Aktion
   * @param prozessID ProzessID
   * @param xmlDatei Name der XML-Datei
   * @param simulationsVariante Simulationsvariante
   */
  private void sendeStartStoppAnfrageV3( String aktion, String prozessID, String xmlDatei, String simulationsVariante )
  {
    AttributeGroup atg = (AttributeGroup) m_connection.getDataModel().getAttributeGroup("atg.startStoppAnfrage");   
    
    Data data = m_connection.createData(atg );
    
    if (data != null)
    {
      data.getReferenceValue( "absenderId" ).setText( new String("" + m_connection.getLocalApplicationObject().getId()) );
      data.getTextValue( "absenderZeichen" ).setText( new String("Absender Zeichen XYZ") );
  
      data.getArray( "V1" ).setLength(0);
      data.getArray( "V2" ).setLength(0);
      data.getArray( "V4" ).setLength(0);
      data.getArray( "V3" ).setLength(1);
      data.getArray( "V3" ).getItem( 0 ).getScaledValue("Aktion").setText( aktion );   
      data.getArray( "V3" ).getItem( 0 ).getTextValue( "StartStoppID").setText(prozessID );
      data.getArray( "V3" ).getItem( 0 ).getScaledValue( "SimulationsVariante" ).set( Long.parseLong(simulationsVariante)  );
      data.getArray( "V3" ).getItem( 0 ).getTextValue( "xmlDatei" ).setText( xmlDatei );
      
      sendeDaten( data );
    } 
  }

  /**
   * Methode zum Senden einer StartStoppAnfrage Variante 4
   * @param aktion Aktion
   * @param prozessID ProzessID
   * @param prozessName Name des Prozesses
   * @param simulationsVariante Simulationsvariante
   */
  private void sendeStartStoppAnfrageV4( String aktion, String prozessID, String prozessName, String simulationsVariante )
  {
    AttributeGroup atg = (AttributeGroup) m_connection.getDataModel().getAttributeGroup("atg.startStoppAnfrage");   
    
    Data data = m_connection.createData(atg );
    
    if (data != null)
    {
      data.getReferenceValue( "absenderId" ).setText( new String("" + m_connection.getLocalApplicationObject().getId()) );
      data.getTextValue( "absenderZeichen" ).setText( new String("Absender Zeichen XYZ") );

      data.getArray( "V1" ).setLength(0);
      data.getArray( "V3" ).setLength(0);
      data.getArray( "V2" ).setLength(0);
      data.getArray( "V4" ).setLength(1);
      data.getArray( "V4" ).getItem( 0 ).getScaledValue("Aktion").setText( aktion );   
      data.getArray( "V4" ).getItem( 0 ).getTextValue( "ProzessID").setText(prozessID);
      data.getArray( "V4" ).getItem( 0 ).getTextValue( "Name").setText ( prozessName );
      
      sendeDaten (data);
    }
  }

  /**
   * Sleep
   * @param ms Millisekunden
   */
  private void mySleep (long ms)
  {
    try
    {
      Thread.sleep (ms);
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    } 
  }

  /**
   * Methode zum Senden der Attributgruppe atg.startStoppAnfrage an den DaV
   * @param data zu sendende Daten
   */
  private void sendeDaten (Data data)
  {
    SystemObject sysObj = m_connection.getDataModel().getObject("rechner.Rechner3");

    AttributeGroup atg = (AttributeGroup) m_connection.getDataModel().getAttributeGroup("atg.startStoppAnfrage");   
    Aspect asp         = m_connection.getDataModel().getAspect("asp.anfrage");
    DataDescription dd = new DataDescription(atg, asp);

    ClientSenderInterface send = new ClientSenderInterface () 
    {
      public void dataRequest( SystemObject object, DataDescription dataDescription, byte state )
      {
      }

      public boolean isRequestSupported( SystemObject object, DataDescription dataDescription )
      {
        return false;
      }
    };
    
    try
    {
      m_connection.subscribeSender ( send, sysObj, dd, SenderRole.sender());
    }
    catch ( OneSubscriptionPerSendData e1 )
    {
      e1.printStackTrace();
    }

    ResultData dr = new ResultData( sysObj, dd, new Date().getTime(), data );

    try
    {
      m_connection.sendData( dr );
    }
    catch ( DataNotSubscribedException e1 )
    {
      e1.printStackTrace();
    }
    catch ( SendSubscriptionNotConfirmed e1 )
    {
      e1.printStackTrace();
    }
    
    // Wieder abmelden
    
    m_connection.unsubscribeSender(send, sysObj, dd);
  }

  /**
   * Methode prft ob die in StartStopp gespeicherten Inkarnationsnamen mit den 
   * Inkarnationsnamen der StartStopp Datei bereinstimmen
   */
  private void pruefeInkarnationsNamen ()
  {
    // Prfen ob Datenstruktur korrekt in StartStopp bernommen wurde
    
    HashMap<String, Boolean> gefunden = new HashMap<String, Boolean> ();
    Iterator<String> it = m_erwarteteNamen.iterator();
    while (it.hasNext())
      gefunden.put( it.next(), false );

    List<ProzessDaten> pd = m_pv.getAlleProzessDaten();
    
    if (pd.size() != m_erwarteteNamen.size())
      fail ("Anzahl Prozesse stimmt nicht !");

    for (int i=0; i<pd.size(); ++i)
      if (gefunden.containsKey( pd.get( i ).getName()) )
        gefunden.put (pd.get( i ).getName(), true);
      else
        fail ("Falscher erwarteter Inkarnationsname: " + pd.get( i ).getName());  
        
    if (gefunden.containsValue( false ))
      fail ("Nicht alle Inkarnationen gefunden !");
  }
}


